import { Door } from './door.js';
import { Player } from './player.js';
import { Room } from './room.js';

function compareBounds(pos: number, lower: number, upper: number) {
	return pos >= lower && pos < upper;
}

export class Game extends HTMLElement {
	private readonly player = new Player();
	private readonly rooms = new Set<Room>();

	private readonly header = document.createElement('header');
	private readonly interactButton = document.createElement('button');

	private keydownCb: (({ code }: KeyboardEvent) => void) | undefined;

	constructor() {
		super();

		this.style.display = 'grid';
		this.style.marginTop = '3rem';
		this.style.marginLeft = '3rem';
		this.style.position = 'relative';

		this.appendChild(this.header);
		this.appendChild(this.interactButton);

		this.interactButton.classList.add('enter-door-display');
		this.interactButton.hidden = true;
	}

	public connectedCallback() {
		this.addControls();
	}

	public disconnectedCallback() {
		this.removeControls();
	}

	public addRoom(room: Room) {
		this.rooms.add(room);
		room.addEventListener('open', () => {
			this.interactButton.onclick = null;
			this.player.parentElement?.removeChild(this.player);
			for (const r of this.rooms) {
				r.parentElement?.removeChild(r);
			}
			room.appendChild(this.player);
			this.appendChild(room);
			this.setHeaderText(room.config.name);
			this.activateAvailableObject();
		});
	}

	public setHeaderText(text: string) {
		this.header.innerText = text.toUpperCase();
	}

	private addControls() {
		this.keydownCb = ({ code }: KeyboardEvent) => {
			switch (code) {
				case 'KeyA':
				case 'ArrowLeft':
					this.player.moveLeft();
					break;
				case 'KeyD':
				case 'ArrowRight':
					this.player.moveRight();
					break;
				case 'KeyW':
				case 'ArrowUp':
					this.player.moveUp();
					break;
				case 'KeyS':
				case 'ArrowDown':
					this.player.moveDown();
					break;
				default:
					return;
			}

			requestAnimationFrame(() => this.activateAvailableObject());
		};

		document.addEventListener('keydown', this.keydownCb);
	}

	private activateAvailableObject() {
		const { axis: { x, y }, dimensions: { height, width } } = this.player;

		let col = 1;
		const posX = x + (width / 2);
		const room = Array.from(this.rooms).find(({ parentElement }) => parentElement === this);

		if (room == null) {
			return;
		}

		for (let i = 1; i <= room.config.rows; i++) {
			const leftBound = (i - 1) * room.area;
			const rightBound = i * room.area;
			if (compareBounds(posX, leftBound, rightBound)) {
				col = i;
				break;
			}
		}

		let row = 1;
		const posY = y + height;
		for (let i = 1; i <= room.config.columns; i++) {
			const topBound = (i - 1) * room.area;
			const bottomBound = i * room.area;
			if (compareBounds(posY, topBound, bottomBound)) {
				row = i;
				break;
			}
		}

		const object = room.getObject(row, col);

		if (object != null) {
			if (this.interactButton.onclick == null) {
				this.interactButton.onclick = (e: KeyboardEvent | MouseEvent) => {
					if (e instanceof KeyboardEvent) {
						const { code } = e;
						if (code === 'KeyE' || code === 'Enter') {
							this.interactButton.hidden = true;
							document.removeEventListener('keydown', this.interactButton.onclick as any);
							object.open();
						}
					} else {
						this.interactButton.hidden = true;
						object.open();
					}
				};
				document.addEventListener('keydown', this.interactButton.onclick as any);
			}
			if (Boolean(this.interactButton.hidden)) {
				this.interactButton.hidden = false;
				this.interactButton.innerText = (object as Door).title || 'View';
			}
		} else {
			document.removeEventListener('keydown', this.interactButton.onclick as any);
			this.interactButton.onclick = null;
			this.interactButton.hidden = true;
		}
	}

	private removeControls() {
		if (this.keydownCb != null) {
			document.removeEventListener('keydown', this.keydownCb);
		}
	}
}
