import { Config } from './config.js';
import { Camera } from './camera.js';
import { Inspectable } from './inspectable.js';

export class Player extends Inspectable {
	private camera = new Camera();

	private _xAxis = 0;
	private _yAxis = 0;

	constructor() {
		super();

		this.style.position = 'absolute';
		this.style.height = '3rem';
		this.style.width = '1rem';

		this.setIcon('player');
	}

	public get axis() {
		return { x: this._xAxis, y: this._yAxis };
	}

	private set xAxis(left: number) {
		const { width } = this.dimensions;
		const { width: boardWidth = 0 } = this.parentElement?.getBoundingClientRect() ?? {};

		if (left + width > boardWidth) {
			left = boardWidth - width;
		} else if (left < 0) {
			left = 0;
		}

		this._xAxis = left;
		this.style.left = `${left}px`;
	}

	private set yAxis(top: number) {
		const { height } = this.dimensions;
		const { height: boardHeight = 0 } = this.parentElement?.getBoundingClientRect() ?? {};

		if (top + height > boardHeight) {
			top = boardHeight - height;
		}

		if (top < 0) {
			top = 0;
		}

		this._yAxis = top;
		this.style.top = `${top}px`;
	}

	public connectedCallback() {
		requestAnimationFrame(() => {
			this.camera.setScreen(this.parentElement as HTMLElement);
			this.moveTo(this._xAxis, this._yAxis);
		});
	}

	public disconnectedCallback() {
		this._xAxis = 0;
		this._yAxis = 0;
		this.camera.reset();
	}

	public cameraShouldFollowVertical() {
		return this.parentElement != null &&
			(
				this._yAxis >= (Config.StepCount * Config.PlayerAcceleration) ||
				this.camera.axis.y < 0
			) &&
			this._yAxis < (this.parentElement.getBoundingClientRect().height / 2);
	}

	public cameraShouldFollowHorizontal() {
		return this.parentElement != null &&
			(
				this._xAxis >= (Config.StepCount * Config.PlayerAcceleration) ||
				this.camera.axis.x < 0
			) &&
			this._xAxis < (this.parentElement.getBoundingClientRect().width / 2);
	}

	public moveLeft(i = Config.StepCount) {
		this.xAxis = this._xAxis - (i * Config.PlayerAcceleration);

		if (this.cameraShouldFollowHorizontal()) {
			this.camera.moveLeft();
		}
	}

	public moveRight(i = Config.StepCount) {
		this.xAxis = this._xAxis + (i * Config.PlayerAcceleration);

		if (this.cameraShouldFollowHorizontal()) {
			this.camera.moveRight();
		}
	}

	public moveUp(i = Config.StepCount) {
		this.yAxis = this._yAxis - (i * Config.PlayerAcceleration);

		if (this.cameraShouldFollowVertical()) {
			this.camera.moveUp();
		}
	}

	public moveDown(i = Config.StepCount) {
		this.yAxis = this._yAxis + (i * Config.PlayerAcceleration);

		if (this.cameraShouldFollowVertical()) {
			this.camera.moveDown();
		}
	}

	public moveTo(x: number, y: number) {
		this.xAxis = x;
		this.yAxis = y;
	}
}
