import { Config, IConfig } from './config.js';
import { Tile } from './tile.js';
import { Inspectable } from './inspectable.js';

export class Room extends HTMLElement {
	private static readonly FRAME_CAP_MS = 16;

	private readonly tiles = new Map<string, Tile>();
	private readonly objects = new Map<string, Inspectable>();

	private _area = 0;
	private lastVal = 50;

	public get area() {
		return this._area;
	}

	constructor(public readonly config: IConfig) {
		super();

		for (let row = 1; row <= this.config.rows; row++) {
			for (let column = 1; column <= this.config.columns; column++) {
				this.addTile(new Tile(row, column));
			}
		}

		this.style.display = 'grid';
		this.style.gridGap = '0';
		this.style.position = 'absolute';
		this.style.animationDirection = '500ms';
		this.style.animationFillMode = 'both';
		this.style.animationName = 'fadeIn';
	}

	public connectedCallback() {
		this.resize(50);
	}

	public open() {
		return this.dispatchEvent(new CustomEvent('open', { cancelable: true }));
	}

	public addTile(tile: Tile) {
		this.tiles.set(`${tile.row}-${tile.column}`, tile);
		this.appendChild(tile);
	}

	public getTile(row: number, column: number) {
		if (row < 1 || row > this.config.rows) {
			throw new RangeError(`Row must be between 1 and ${this.config.rows} (Current: ${row})`);
		}

		if (column < 1 || column > this.config.columns) {
			throw new RangeError(`Column must be between 1 and ${this.config.columns} (Current: ${column})`);
		}

		return this.tiles.get(`${row}-${column}`) as Tile;
	}

	public addObject(row: number, column: number, object: Inspectable) {
		const tile = this.getTile(row, column);

		if (tile.children.length > 0) {
			throw new Error('Tile can only have one object');
		}

		this.objects.set(`${row}-${column}`, tile.appendChild(object));
	}

	public resize(v: number) {
		this._area = v * Config.TileArea;
		const initial = this.lastVal * Config.TileArea;

		if (v > this.lastVal) {
			for (let i = initial, j = 0; i < this._area; i++, j++) {
				this.resizeBoard(i, j);
			}
		} else if (v < this.lastVal) {
			for (let i = initial, j = 0; i > this._area; i--, j++) {
				this.resizeBoard(i, j);
			}
		} else {
			this.resizeBoard(this._area);
		}

		this.lastVal = v;
	}

	private resizeBoard(i: number, j = 0) {
		setTimeout(() => {
			requestAnimationFrame(() => {
				this.style.gridTemplateColumns = `repeat(${this.config.columns}, ${i}px)`;
				this.style.gridTemplateRows = `repeat(${this.config.rows}, ${i}px)`;
			});
		}, j * Room.FRAME_CAP_MS);
	}

	public getObject(row: number, col: number) {
		return this.objects.get(`${row}-${col}`)
			|| this.objects.get(`${row + 1}-${col}`)
			|| this.objects.get(`${row - 1}-${col}`)
			|| this.objects.get(`${row}-${col + 1}`)
			|| this.objects.get(`${row}-${col - 1}`)
			|| this.objects.get(`${row + 1}-${col + 1}`)
			|| this.objects.get(`${row + 1}-${col - 1}`)
			|| this.objects.get(`${row - 1}-${col + 1}`)
			|| this.objects.get(`${row - 1}-${col - 1}`);
	}
}
